program SingleDoubleExtended;
{$APPTYPE CONSOLE}

uses
  Windows,
  SysUtils,
  Profiler in '..\..\Units\Profiler.pas',
  TestCpu in '..\..\Units\TestCPU.pas';

const
  N = 10000000;

var
  xs, ys: Single;
  xd, yd: Double;
  xe, ye: Extended;
  I: Integer;
  Ptr: Pointer;
  Str: String;

begin
  Writeln('Measure speed of floating point types (FPU calculations)'#13#10'(c) Eduard I. Vatutin'#13#10'  ICQ: 203-229-391'#13#10'  e-mail: evatutin@rambler.ru'#13#10'  http://evatutin.narod.ru'#13#10);

  {    TSC }
  SetThreadAffinityMask(GetCurrentThread(), $0001);

  GetMem(Ptr, 100);     {  ... }
  Integer(Ptr) := (Integer(Ptr) + 16) and not 16;   {    16 }

  TestCpuid();
  Writeln('CPU ', CpuInfo.Vendor + '.' + IntToHex(CpuInfo.VerInfo, 5), #13#10);

  Writeln('Single / Double / Extended (clocks)'#13#10);

  { ================================================================================================================================== }
  Write('1. Unaligned variables (typical for Delphi compiler) ');
  xs := 0.1;
  ys := 0.0;

  StartProf();
    for I := 1 to N do
      ys := ys + xs;
  StopProf();
  Str := FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  xd := 0.1;
  yd := 0.0;

  StartProf();
    for I := 1 to N do
      yd := yd + xd;
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(yd, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                      '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  xe := 0.1;
  ye := 0.0;

  StartProf();
    for I := 1 to N do
      ye := ye + xe;
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2);
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  Writeln(Str);

  { ================================================================================================================================== }
  Write('2. Aligned variables, change PC (addition)         ');

  { Single }
  Set8087CW((Get8087CW() and  $FCFF) or $0000);     // $0000 - single, $0200 - double, $0300 - extended
  Single(Ptr^) := 0.1;
  Single(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Single(Pointer(Integer(Ptr)+16)^) := Single(Pointer(Integer(Ptr)+16)^) + Single(Ptr^);
  StopProf();
  Str := FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Double }
  Set8087CW((Get8087CW() and  $FCFF) or $0200);
  Double(Ptr^) := 0.1;
  Double(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Double(Pointer(Integer(Ptr)+16)^) := Double(Pointer(Integer(Ptr)+16)^) + Double(Ptr^);
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Extended }
  Set8087CW((Get8087CW() and  $FCFF) or $0300);
  Extended(Ptr^) := 0.1;
  Extended(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Extended(Pointer(Integer(Ptr)+16)^) := Extended(Pointer(Integer(Ptr)+16)^) + Extended(Ptr^);
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2);
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  Writeln(Str);

  { ================================================================================================================================== }
  Write('3. Aligned variables, change PC (multiplication)  ');

  { Single }
  Set8087CW((Get8087CW() and  $FCFF) or $0000);
  Single(Ptr^) := 0.1;
  Single(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Single(Pointer(Integer(Ptr)+16)^) := Single(Pointer(Integer(Ptr)+16)^) * Single(Ptr^);
  StopProf();
  Str := FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Double }
  Set8087CW((Get8087CW() and  $FCFF) or $0100);
  Double(Ptr^) := 0.1;
  Double(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Double(Pointer(Integer(Ptr)+16)^) := Double(Pointer(Integer(Ptr)+16)^) * Double(Ptr^);
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Extended }
  Set8087CW((Get8087CW() and  $FCFF) or $0300);  
  Extended(Ptr^) := 0.1;
  Extended(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Extended(Pointer(Integer(Ptr)+16)^) := Extended(Pointer(Integer(Ptr)+16)^) * Extended(Ptr^);
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2);
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  Writeln(Str);

  { ================================================================================================================================== }
  Write('4. Aligned variables, change PC (division)        ');

  { Single }
  Set8087CW((Get8087CW() and  $FCFF) or $0000);
  Single(Ptr^) := 0.1;
  Single(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Single(Pointer(Integer(Ptr)+16)^) := Single(Pointer(Integer(Ptr)+16)^) + 1 / Single(Ptr^);
  StopProf();
  Str := FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Double }
  Set8087CW((Get8087CW() and  $FCFF) or $0200);
  Double(Ptr^) := 0.1;
  Double(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Double(Pointer(Integer(Ptr)+16)^) := Double(Pointer(Integer(Ptr)+16)^) + 1 / Double(Ptr^);
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Extended }
  Set8087CW((Get8087CW() and  $FCFF) or $0300);
  Extended(Ptr^) := 0.1;
  Extended(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Extended(Pointer(Integer(Ptr)+16)^) := Extended(Pointer(Integer(Ptr)+16)^) + 1 / Extended(Ptr^);
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2);
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  Writeln(Str);

  { ================================================================================================================================== }
  Write('5. Aligned variables, change PC (square root)     ');

  { Single }
  Set8087CW((Get8087CW() and  $FCFF) or $0000);
  Single(Ptr^) := 0.1;
  Single(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Single(Pointer(Integer(Ptr)+16)^) := Single(Pointer(Integer(Ptr)+16)^) + sqrt( Single(Ptr^) );
  StopProf();
  Str := FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Double }
  Set8087CW((Get8087CW() and  $FCFF) or $0200);
  Double(Ptr^) := 0.1;
  Double(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Double(Pointer(Integer(Ptr)+16)^) := Double(Pointer(Integer(Ptr)+16)^) + sqrt( Double(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Extended }
  Set8087CW((Get8087CW() and  $FCFF) or $0300);
  Extended(Ptr^) := 0.1;
  Extended(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Extended(Pointer(Integer(Ptr)+16)^) := Extended(Pointer(Integer(Ptr)+16)^) + sqrt( Extended(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2);
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  Writeln(Str);

  { ================================================================================================================================== }
  Write('6. Aligned variables, change PC (sin)            ');

  { Single }
  Set8087CW((Get8087CW() and  $FCFF) or $0000);
  Single(Ptr^) := 0.1;
  Single(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Single(Pointer(Integer(Ptr)+16)^) := Single(Pointer(Integer(Ptr)+16)^) + sin( Single(Ptr^) );
  StopProf();
  Str := FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Double }
  Set8087CW((Get8087CW() and  $FCFF) or $0200);
  Double(Ptr^) := 0.1;
  Double(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Double(Pointer(Integer(Ptr)+16)^) := Double(Pointer(Integer(Ptr)+16)^) + sin( Double(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Extended }
  Set8087CW((Get8087CW() and  $FCFF) or $0300);
  Extended(Ptr^) := 0.1;
  Extended(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Extended(Pointer(Integer(Ptr)+16)^) := Extended(Pointer(Integer(Ptr)+16)^) + sin( Extended(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2);
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  Writeln(Str);

  { ================================================================================================================================== }
  Write('7. Aligned variables, change PC (arctan)         ');

  { Single }
  Set8087CW((Get8087CW() and  $FCFF) or $0000);
  Single(Ptr^) := 0.1;
  Single(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Single(Pointer(Integer(Ptr)+16)^) := Single(Pointer(Integer(Ptr)+16)^) + arctan( Single(Ptr^) );
  StopProf();
  Str := FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Double }
  Set8087CW((Get8087CW() and  $FCFF) or $0200);
  Double(Ptr^) := 0.1;
  Double(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Double(Pointer(Integer(Ptr)+16)^) := Double(Pointer(Integer(Ptr)+16)^) + arctan( Double(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Extended }
  Set8087CW((Get8087CW() and  $FCFF) or $0300);
  Extended(Ptr^) := 0.1;
  Extended(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Extended(Pointer(Integer(Ptr)+16)^) := Extended(Pointer(Integer(Ptr)+16)^) + arctan( Extended(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2);
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  Writeln(Str);

  { ================================================================================================================================== }
  Write('8. Aligned variables, change PC (exp)            ');

  { Single }
  Set8087CW((Get8087CW() and  $FCFF) or $0000);
  Single(Ptr^) := 0.1;
  Single(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Single(Pointer(Integer(Ptr)+16)^) := Single(Pointer(Integer(Ptr)+16)^) + exp( Single(Ptr^) );
  StopProf();
  Str := FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Double }
  Set8087CW((Get8087CW() and  $FCFF) or $0200);
  Double(Ptr^) := 0.1;
  Double(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Double(Pointer(Integer(Ptr)+16)^) := Double(Pointer(Integer(Ptr)+16)^) + exp( Double(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Extended }
  Set8087CW((Get8087CW() and  $FCFF) or $0300);
  Extended(Ptr^) := 0.1;
  Extended(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Extended(Pointer(Integer(Ptr)+16)^) := Extended(Pointer(Integer(Ptr)+16)^) + exp( Extended(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2);
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  Writeln(Str);

  { ================================================================================================================================== }
  Write('9. Aligned variables, change PC (ln)              ');

  { Single }
  Set8087CW((Get8087CW() and  $FCFF) or $0000);
  Single(Ptr^) := 0.1;
  Single(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Single(Pointer(Integer(Ptr)+16)^) := Single(Pointer(Integer(Ptr)+16)^) + ln( Single(Ptr^) );
  StopProf();
  Str := FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Double }
  Set8087CW((Get8087CW() and  $FCFF) or $0200);
  Double(Ptr^) := 0.1;
  Double(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Double(Pointer(Integer(Ptr)+16)^) := Double(Pointer(Integer(Ptr)+16)^) + ln( Double(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2) + ' / ';
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  { Extended }
  Set8087CW((Get8087CW() and  $FCFF) or $0300);
  Extended(Ptr^) := 0.1;
  Extended(Pointer(Integer(Ptr)+16)^) := 0.0;

  StartProf();
    for I := 1 to N do
      Extended(Pointer(Integer(Ptr)+16)^) := Extended(Pointer(Integer(Ptr)+16)^) + ln( Extended(Ptr^) );
  StopProf();
  Str := Str + FloatToStrF(ProfResult()/N, ffFixed, 6, 2);
  Write(ys, #8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8'                       '#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8#8);

  Writeln(Str);

  Readln;
end.
